/**
 * Created by wilsonshang on 16-7-20.
 */

/**
 * @author wilson
 * @description 日历"月"元素
 * @constructor
 * @param {UI.DatePanel} datePanel - 父类面板引用
 * @param {int} year - 年;例如:2017
 * @param {int} month - 月
 * @param {int} todayTimestamp - 今天的时间戳
 *  @param {int} days - 总共只能可选的天数
 * */
UI.MonthPanel = function (datePanel, year, month, todayTimestamp, days) {

    UI.ListItem.call(this);

    this.dom.className = "MonthPanel";

    this.datePanel = datePanel;
    this.year = year;
    this.month = month;

    var ymTxt = new UI.Text(year + "年" + (month+1) + "月");
    this.add(ymTxt);
    ymTxt.addClass("month-label");

    this.add(new UI.HorizontalRule());


    //该月有多少天
    this.dayAmount = DateUtils.getDaysForMonth(year, month);

    //可选的天数
    if(days){
        this.days = days;
    }else {
        this.days = 730;
    }



    this.allItems = [];

    var date, day, item, timeStamp, endTimeStamp;

    var flag = true;

    for (var j = 1; j <= this.dayAmount; ++j) {
        date = new Date(year, month, j);

        day = date.getDay();

        item = new UI.DateItem(this, {date: date});

        this.add(item);

        this.allItems.push(item);

        item.setMarginBottom("6px");

        if (j === 1) {

            item.setHorizontalOffset(day);

        }

        timeStamp = date.getTime();

        endTimeStamp = (new Date()).getTime() + this.days * 24 * 3600 * 1000 ;

        item.setDisabled(timeStamp < todayTimestamp || timeStamp > endTimeStamp);

        if (day == 6) {

            if (j === this.dayAmount) flag = false;
            this.add(new UI.HorizontalRule());

        }
    }

    if (flag)  {

        this.add(new UI.HorizontalRule());
    }


    this.selectedDateMin = -1;
    this.selectedDateMax = -1;

};

UI.MonthPanel.prototype = Object.create(UI.Panel.prototype);

Object.assign(UI.MonthPanel.prototype, {

    constructor: UI.MonthPanel,

    /**
     * @description 点击传递函数
     * @param {UI.DateItem} dateItem - 点击事件目标源
     * */
    tapHandler: function (dateItem) {

        var result = this.datePanel.tapHandler(dateItem);

    },

    /**
     * @description 根据日期获得对应的日历可视对象
     * @param {Date} date - 日期
     * @return {UI.DateItem} dateItem - 点击事件对象
     * */
    getDateItem: function (date) {

        var d = date.getDate();
        return this.allItems[d - 1];

    },

    /**
     * @description 设置该月内日期内(闭区间)的日历组件,并将其设置为选择状态
     * @param {int} dateMin - 日期起始点
     * @param {int} date - 日期终结点
     * @return {array} r - 日历组件列表
     * */
    setItemsSelected: function (dateMin, dateMax) {

        this.selectedDateMin = dateMin;

        this.selectedDateMax = dateMax;

        var r = [];

        for (var i = dateMin - 1; i < dateMax; ++i) {

            if (i === dateMin - 1) {

                r.push(this.allItems[i]);

            }

            if (i === dateMax - 1) {

                r.push(this.allItems[i]);

            }

            this.allItems[i].setStatus(DateUtils.DATE_ITEM_SELECTED);

        }


        return r;

    },

    /**
     * @description 清除该月选择状态
     * @return {UI.MonthPanel} 自身
     * */
    clearSelect: function () {

        if (this.selectedDateMin === -1 || this.selectedDateMax === -1) return false;

        for (var i = this.selectedDateMin - 1; i < this.selectedDateMax; ++i) {

            this.allItems[i].setStatus(DateUtils.DATE_ITEM_NORMAL);
            this.allItems[i].setInfo("");

        }
        return this;

    }

});